-- Mass-select people from the invite list to modify their statuses
-- 15

local core = CalendarExtras

local module = core.NewModule("Mass Select")

 --[[ -------- Locals ------- ]]--

local selectedByName = {}
local selectedByIndex = setmetatable({}, {__newindex = function(self, key, value) -- Generate name-table automatically from the index-table
	rawset(self, key, value)
	if value ~= nil then
		selectedByName[value] = true
	elseif key then
		local name = CalendarEventGetInvite(key)
		if name then
			selectedByName[name] = nil
		end
	end
end})
local lastSelectedIndex, lastSelectedName = 1, nil
local listIsDirty

local currentInviteList -- The currently open scrollframe (CalendarViewEventInviteListScrollFrame or CalendarCreateEventInviteListScrollFrame)

local highlightTexture

-- Our functions
local Select, Unselect, UnselectAll, RangeSelect
local OnClick, RebuildSelectedIndices, UpdateInviteListButtons

local sortersModule

 --[[ ----------------------- ]]--

function Select(index)
	selectedByIndex[index] = CalendarEventGetInvite(index)
end

function Unselect(index)
	selectedByIndex[index] = nil
end

function UnselectAll()
	wipe(selectedByIndex)
	wipe(selectedByName)
end

function RangeSelect(from, to)
	local buttons = currentInviteList.buttons
	local funkySorting = sortersModule and #sortersModule.reSortedList > 0 and sortersModule.reSortedList
	if funkySorting then
		local foundF, foundT
		for k, v in ipairs(funkySorting) do
			if not foundF and v == from then
				from = k
				foundF = true
			elseif not foundT and v == to then
				to = k
				foundT = true
			end
			if foundF and foundT then break end
		end
	end
	for i = from, to, from > to and -1 or 1 do
		Select(funkySorting and funkySorting[i] or i)
	end
end

function OnClick(self, button)
	if button == "LeftButton" then
		if IsShiftKeyDown() and IsControlKeyDown() then -- Add range select
			RangeSelect(lastSelectedIndex, self.inviteIndex)
		elseif IsShiftKeyDown() then -- Range select
			UnselectAll()
			RangeSelect(lastSelectedIndex, self.inviteIndex)
		else
			if IsControlKeyDown() then -- Add to selection
				if selectedByIndex[self.inviteIndex] then
					Unselect(self.inviteIndex)
					self:GetHighlightTexture():SetTexture() -- Clear the highlight texture temporarily so the button under the cursor unhighlights when unselected
				else
					Select(self.inviteIndex)
				end
			else -- Single select
				UnselectAll()
				Select(self.inviteIndex)
			end
			lastSelectedIndex, lastSelectedName = self.inviteIndex, CalendarEventGetInvite(self.inviteIndex)
		end
	elseif button == "RightButton" then
		if not selectedByIndex[self.inviteIndex] then -- Unselect everything if we didn't right-click something that was already selected
			UnselectAll()
		end
		Select(self.inviteIndex)
		lastSelectedIndex, lastSelectedName = self.inviteIndex, CalendarEventGetInvite(self.inviteIndex)
	end
	UpdateInviteListButtons()
end

function OnLeave(self)
	self:GetHighlightTexture():SetTexture(highlightTexture)
	GameTooltip:Hide()
end

-- Rebuild the selectedByIndex table from selectedByName when the indices change, i.e. when the list is re-sorted or the list otherwise changes
function RebuildSelectedIndices()
	if not next(selectedByName) then return end
	wipe(selectedByIndex)
	local i, name = 1
	while true do
		name = CalendarEventGetInvite(i)
		if not name then break end
		if selectedByName[name] then
			selectedByIndex[i] = name
		end
		if name == lastSelectedName then
			lastSelectedIndex = i
		end
		i = i + 1
	end
	UpdateInviteListButtons() -- This is kinda stupid, but the list update hooks fire before the "CALENDAR_UPDATE_INVITE_LIST" event reaches this module >_>
end

-- Update the highlights on the visible buttons according to selection
function UpdateInviteListButtons(_, frame)
	if listIsDirty then
		listIsDirty = nil
		RebuildSelectedIndices()
	end
	if frame then currentInviteList = frame end
	for _, button in ipairs(currentInviteList.buttons) do
		if button.inviteIndex and selectedByIndex[button.inviteIndex] then
			button:LockHighlight()
		else
			button:UnlockHighlight()
		end
	end
end

-- Let other modules etc run functions on the selected stuff
function module.RunOnSelected(func, useName, value)
	for index in pairs(useName and selectedByName or selectedByIndex) do
		func(index, value)
	end
end

function module.OnInitialize()
	if core.modules.Sorters then
		sortersModule = core.modules.Sorters
	end

	-- Hook OnClicks so we start selecting stuff on left-click
	module:Hook("CalendarViewEventInviteListButton_OnClick", OnClick, true)
	module:Hook("CalendarCreateEventInviteListButton_OnClick", OnClick, true)

	core.RegisterCallback(module, "OpenEvent", function(event, frame)
		currentInviteList = frame
		lastSelectedIndex, lastSelectedName = 1, nil
		UnselectAll()
	end)

	core.RegisterCallback(module, "Post_Update", UpdateInviteListButtons)

	core.RegisterCallback(module, "CustomResorted", UpdateInviteListButtons)

	module:RegisterEvent("CALENDAR_UPDATE_INVITE_LIST", function()
		listIsDirty = true
	end)

	highlightTexture = CalendarViewEventInviteList.scrollFrame.buttons[1]:GetHighlightTexture():GetTexture()
	for _, button in ipairs(CalendarViewEventInviteList.scrollFrame.buttons) do
		button:SetScript("OnLeave", OnLeave)
	end
	for _, button in ipairs(CalendarCreateEventInviteList.scrollFrame.buttons) do
		button:SetScript("OnLeave", OnLeave)
	end

	-- Hook the right-click menu functions to make them apply to all of the selected players
	module:RawHook("CalendarInviteContextMenu_RemoveInvite", true)
	module:RawHook("CalendarInviteContextMenu_SetModerator", true)
	module:RawHook("CalendarInviteContextMenu_ClearModerator", true)
	module:RawHook("CalendarInviteContextMenu_InviteToGroup", true)
	module:RawHook("CalendarInviteStatusContextMenu_SetStatusOption", true)
	-- Override the default context-menu highlight-killer, we don't want that
	module:RawHook("CalendarInviteContextMenu_UnlockHighlights", function() end, true)
end

 --[[ -------- Hooks -------- ]]--

function module:CalendarInviteContextMenu_RemoveInvite()
	for index in pairs(selectedByIndex) do
		CalendarEventRemoveInvite(index)
	end
end

function module:CalendarInviteContextMenu_SetModerator()
	for index in pairs(selectedByIndex) do
		CalendarEventSetModerator(index)
	end
end

function module:CalendarInviteContextMenu_ClearModerator()
	for index in pairs(selectedByIndex) do
		CalendarEventClearModerator(index)
	end
end

function module:CalendarInviteContextMenu_InviteToGroup()
	for k in pairs(selectedByName) do
		InviteUnit(k)
	end
end

function module:CalendarInviteStatusContextMenu_SetStatusOption(menu)
	for index in pairs(selectedByIndex) do
		CalendarEventSetStatus(index, menu.value)
	end
	CalendarContextMenu_Hide(CalendarCreateEventInviteContextMenu_Initialize)
end